using System;
using System.Configuration;
using System.Configuration.Provider;
using System.Web.Configuration;

namespace Quotations
{
    public static class QuoteManager
    {
        //Initialization related variables and logic
        //private static bool isInitialized = false;
        //private static Exception initializationException;

        //private static object initializationLock = new object();

        static QuoteManager()
        {
            Initialize();
        }

        private static void Initialize()
        {
       
            try
            {
                //Get the feature's configuration info
                QuotationsConfiguration qc =
                    (QuotationsConfiguration)ConfigurationManager.GetSection("quotations");

                if (qc.DefaultProvider == null || qc.Providers == null || qc.Providers.Count < 1)
                    throw new ProviderException("You must specify a valid default provider.");

                //Instantiate the providers
                providerCollection = new QuotationsProviderCollection();
                ProvidersHelper.InstantiateProviders(qc.Providers, providerCollection, typeof(QuotationsProvider));
                providerCollection.SetReadOnly();
                defaultProvider = providerCollection[qc.DefaultProvider];
                if (defaultProvider == null)
                {
                    throw new ConfigurationErrorsException(
                        "You must specify a default provider for the feature.",
                        qc.ElementInformation.Properties["defaultProvider"].Source,
                        qc.ElementInformation.Properties["defaultProvider"].LineNumber);
                }
            }
            catch (Exception ex)
            {
                //initializationException = ex;
                //isInitialized = true;
                throw ex;
            }

            //isInitialized = true; //error-free initialization
        }

        //Public feature API
        private static QuotationsProvider defaultProvider;
        private static QuotationsProviderCollection providerCollection;

        public static QuotationsProvider Provider
        {
            get 
            {
                return defaultProvider; 
            }
        }

        public static QuotationsProviderCollection Providers
        {
            get
            {
                return providerCollection;
            }
        }

        public static string GetQuoteOfTheDay()
        {
            return Provider.GetQuoteOfTheDay();
        }
    }
}
