using System;
using System.Collections.Specialized;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Configuration.Provider;
using System.Web;
using System.Web.Hosting;
using System.Xml;

namespace EnvironmentConfigurationProvider
{
    public class EnvironmentConfigurationProvider : ProtectedConfigurationProvider
    {
        private string connectionString = "server=.;database=aspnetdb;Integrated Security=true";

        public override XmlNode Encrypt(XmlNode node)
        {
            throw new NotSupportedException("This provider only supports decrypt operations.");
        }

        public override XmlNode Decrypt(XmlNode encryptedNode)
        {
            //Determine the machine name
            string machineName = Environment.GetEnvironmentVariable("COMPUTERNAME");

            //Determine the application name and truncates the extension in the case of a non-ASP.NET application
            string applicationName = HostingEnvironment.ApplicationVirtualPath;
            if (String.IsNullOrEmpty(applicationName))
            {
                applicationName = System.Diagnostics.Process.GetCurrentProcess().MainModule.ModuleName;

                int indexOfDot = applicationName.IndexOf('.');
                if (indexOfDot != -1)
                {
                    applicationName = applicationName.Remove(indexOfDot);
                }
            }

            XmlNode n = encryptedNode.SelectSingleNode("/EncryptedData/configInfo");
            string featureName = n.Attributes["featureName"].Value;

            using (SqlConnection conn = new SqlConnection(connectionString))
            {
                SqlCommand cmd = new SqlCommand("GetConfiguration", conn);
                cmd.CommandType = CommandType.StoredProcedure;
                SqlParameter p1 = new SqlParameter("@pMachineName", machineName);
                SqlParameter p2 = new SqlParameter("@pApplicationName", applicationName);
                SqlParameter p3 = new SqlParameter("@pFeatureName", featureName);
                cmd.Parameters.AddRange(new SqlParameter[] { p1, p2, p3 });

                conn.Open();
                string configText = (string)cmd.ExecuteScalar();

                //Convert string from the database into an XmlNode
                XmlDocument xd = new XmlDocument();
                xd.LoadXml(configText);

                return xd.DocumentElement;
            }
        }

        public override void Initialize(string name, NameValueCollection config)
        {
            if ((config == null) || (config.Count == 0))
                throw new ArgumentNullException("You must supply a valid configuration dictionary.");

            //Let ProviderBase perform the basic initialization
            base.Initialize(name, config);

            //Get the connection string from configuration
            //string connectionStringName = config["connectionStringName"];
            //if (String.IsNullOrEmpty(connectionStringName))
            //    throw new ProviderException("You must specify connectionStringName in the provider configuration");

            //ConnectionStringsSection cs =
            //    (ConnectionStringsSection)ConfigurationManager.GetSection("connectionStrings");
            //if (cs == null)
            //    throw new ProviderException("An error occurred retrieving the connection strings section.");

            //if (cs.ConnectionStrings[connectionStringName] == null)
            //    throw new ProviderException("The connection string could not be found in the connection strings section.");
            //else
            //    connectionString = cs.ConnectionStrings[connectionStringName].ConnectionString;

            //if (String.IsNullOrEmpty(connectionString))
            //    throw new ProviderException("The connection string is invalid.");

            //Done processing the conection string
            config.Remove("connectionStringName");

            //Check to see if unexpected attributes were set in configuration
            if (config.Count > 0)
            {
                string extraAttribute = config.GetKey(0);
                if (!String.IsNullOrEmpty(extraAttribute))
                    throw new ProviderException("The following unrecognized attribute was found in " + Name + "'s configuration: '" +
                                                extraAttribute + "'");
                else
                    throw new ProviderException("An unrecognized attribute was found in the provider's configuration.");
            }
        }
    }
}
