using System;
using System.Configuration;
using System.Configuration.Provider;
using System.Collections.Generic;
using System.Text;
using System.Web.Security;
using MembershipWebserviceProvider.MembershipWebServiceEndpoint;

namespace MembershipWebserviceProvider
{
    public class MembershipWebserviceProvider : MembershipProvider
    {
        private string applicationName;
        private string connectionString;

        public override string ApplicationName
        {
            get { return applicationName; }
            set { applicationName = value; }
        }

        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            if ((config == null) || (config.Count == 0))
                throw new ArgumentNullException("You must supply a valid configuration dictionary.");

            if (string.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "localized description here");
            }

            //Let ProviderBase perform the basic initialization
            base.Initialize(name, config);

            //Perform feature-specific provider initialization here
            //"Application" name in this provider will mean provider name
            applicationName = config["applicationName"];
            if (string.IsNullOrEmpty(applicationName))
                applicationName = "AspNetSqlMembershipProvider";
            config.Remove("applicationName");

            //Get the connection string
            string connectionStringName = config["connectionStringName"];
            if (String.IsNullOrEmpty(connectionStringName))
                throw new ProviderException("You must specify a connectionStringName attribute.");

            ConnectionStringsSection cs =
                (ConnectionStringsSection)ConfigurationManager.GetSection("connectionStrings");
            if (cs == null)
                throw new ProviderException("An error occurred retrieving the connection strings section.");

            if (cs.ConnectionStrings[connectionStringName] == null)
                throw new ProviderException("The connection string could not be found in the connection strings section.");
            else
                connectionString = cs.ConnectionStrings[connectionStringName].ConnectionString;

            if (String.IsNullOrEmpty(connectionString))
                throw new ProviderException("The connection string is invalid.");
            config.Remove("connectionStringName");

            //Check to see if unexpected attributes were set in configuration
            if (config.Count > 0)
            {
                string extraAttribute = config.GetKey(0);
                if (!String.IsNullOrEmpty(extraAttribute))
                    throw new ProviderException("The following unrecognized attribute was found in " + Name + "'s configuration: '" +
                                                extraAttribute + "'");
                else
                    throw new ProviderException("An unrecognized attribute was found in the provider's configuration.");
            }
        }

        public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
        {
            WebServiceFormsTicket wt = new WebServiceFormsTicket();
            wt.Ticket = null;
            wt.ProviderName = ApplicationName;

            MembershipWebService ws = new MembershipWebService();
            ws.WebServiceFormsTicketValue = wt;
            ws.Url = this.connectionString;

            try
            {
                WebServiceProxyableMembershipUser wsu = ws.CreateUser(username, password, email, passwordQuestion,
                                                                  passwordAnswer, isApproved, providerUserKey);

                status = MembershipCreateStatus.Success;
                return new MembershipUser(this.Name, wsu.userName, wsu.providerUserKey, wsu.email, wsu.passwordQuestion,
                                          wsu.comment, wsu.isApproved, wsu.isLockedOut, wsu.creationDate, wsu.lastLoginDate,
                                          wsu.lastActivityDate, wsu.lastPasswordChangedDate, wsu.lastLockoutDate);
            }
            catch
            {
                status = MembershipCreateStatus.ProviderError;
                return null;
            }
        }

        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            WebServiceFormsTicket wt = new WebServiceFormsTicket();
            wt.Ticket = null;
            wt.ProviderName = ApplicationName;

            MembershipWebService ws = new MembershipWebService();
            ws.WebServiceFormsTicketValue = wt;
            ws.Url = this.connectionString;

            try
            {
                WebServiceProxyableSearchResult retval = ws.FindUsersByName(usernameToMatch, pageIndex, pageSize);
                MembershipUserCollection muc = new MembershipUserCollection();
                totalRecords = retval.TotalRecords;
                foreach (WebServiceProxyableMembershipUser wsu in retval.SearchResult)
                {
                    muc.Add(
                        new MembershipUser(this.Name, wsu.userName, wsu.providerUserKey, wsu.email, wsu.passwordQuestion,
                                              wsu.comment, wsu.isApproved, wsu.isLockedOut, wsu.creationDate, wsu.lastLoginDate,
                                              wsu.lastActivityDate, wsu.lastPasswordChangedDate, wsu.lastLockoutDate)
                           );
                }

                return muc;
            }
            catch
            {
                throw;
            }
        }

        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
            WebServiceFormsTicket wt = new WebServiceFormsTicket();
            wt.Ticket = null;
            wt.ProviderName = ApplicationName;

            MembershipWebService ws = new MembershipWebService();
            ws.WebServiceFormsTicketValue = wt;
            ws.Url = this.connectionString;

            try
            {
                WebServiceProxyableMembershipUser wsu = ws.GetUser(username, userIsOnline);
                if (wsu != null)
                {
                    return new MembershipUser(this.Name, wsu.userName, wsu.providerUserKey, wsu.email, wsu.passwordQuestion,
                                          wsu.comment, wsu.isApproved, wsu.isLockedOut, wsu.creationDate, wsu.lastLoginDate,
                                          wsu.lastActivityDate, wsu.lastPasswordChangedDate, wsu.lastLockoutDate);
                }
                else
                    return null;
            }
            catch
            {
                throw;
            }
        }

        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
            WebServiceFormsTicket wt = new WebServiceFormsTicket();
            wt.Ticket = null;
            wt.ProviderName = ApplicationName;

            MembershipWebService ws = new MembershipWebService();
            ws.WebServiceFormsTicketValue = wt;
            ws.Url = this.connectionString;

            try
            {
                WebServiceProxyableSearchResult retval = ws.GetAllUsers(pageIndex, pageSize);
                MembershipUserCollection muc = new MembershipUserCollection();
                totalRecords = retval.TotalRecords;
                foreach (WebServiceProxyableMembershipUser wsu in retval.SearchResult)
                {
                    muc.Add(
                        new MembershipUser(this.Name, wsu.userName, wsu.providerUserKey, wsu.email, wsu.passwordQuestion,
                                              wsu.comment, wsu.isApproved, wsu.isLockedOut, wsu.creationDate, wsu.lastLoginDate,
                                              wsu.lastActivityDate, wsu.lastPasswordChangedDate, wsu.lastLockoutDate)
                           );
                }

                return muc;
            }
            catch
            {
                throw;
            }
        }

        public override void UpdateUser(MembershipUser user)
        {
            WebServiceFormsTicket wt = new WebServiceFormsTicket();
            wt.Ticket = null;
            wt.ProviderName = ApplicationName;

            MembershipWebService ws = new MembershipWebService();
            ws.WebServiceFormsTicketValue = wt;
            ws.Url = this.connectionString;

            WebServiceProxyableMembershipUser wsu = new WebServiceProxyableMembershipUser();
            wsu.comment = user.Comment;
            wsu.creationDate = user.CreationDate;
            wsu.email = user.Email;
            wsu.isApproved = user.IsApproved;
            wsu.isLockedOut = user.IsLockedOut;
            wsu.isOnline = user.IsOnline;
            wsu.lastActivityDate = user.LastActivityDate;
            wsu.lastLockoutDate = user.LastLockoutDate;
            wsu.lastLoginDate = user.LastLoginDate;
            wsu.lastPasswordChangedDate = user.LastPasswordChangedDate;
            wsu.passwordQuestion = user.PasswordQuestion;
            wsu.providerName = this.ApplicationName;
            wsu.providerUserKey = user.ProviderUserKey.ToString();
            wsu.userName = user.UserName;

            try
            {
                ws.UpdateUser(wsu);
            }
            catch
            {
                throw;
            }
        }

        public override bool ValidateUser(string username, string password)
        {
            WebServiceFormsTicket wt = new WebServiceFormsTicket();
            wt.Ticket = null;
            wt.ProviderName = ApplicationName;

            MembershipWebService ws = new MembershipWebService();
            ws.WebServiceFormsTicketValue = wt;
            ws.Url = this.connectionString;

            try
            {
                return ws.ValidateUser(username, password);
            }
            catch
            {
                return false;
            }
        }

        //
        //
        //
        //
        //Default stubbed out implementations for other methods and properties
        public override bool ChangePassword(string username, string oldPassword, string newPassword)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override bool EnablePasswordReset
        {
            get { return false; }
        }

        public override bool EnablePasswordRetrieval
        {
            get { return false;  }
        }

        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override int GetNumberOfUsersOnline()
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override string GetPassword(string username, string answer)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override string GetUserNameByEmail(string email)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override int MaxInvalidPasswordAttempts
        {
            get { return 0; }
        }

        public override int MinRequiredNonAlphanumericCharacters
        {
            get { return 0; }
        }

        public override int MinRequiredPasswordLength
        {
            get { return 0; }
        }

        public override int PasswordAttemptWindow
        {
            get { return 0; }
        }

        public override MembershipPasswordFormat PasswordFormat
        {
            get { return MembershipPasswordFormat.Hashed; }
        }

        public override string PasswordStrengthRegularExpression
        {
            get { return null; }
        }

        public override bool RequiresQuestionAndAnswer
        {
            get { return false; }
        }

        public override bool RequiresUniqueEmail
        {
            get { return true; }
        }

        public override string ResetPassword(string username, string answer)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

        public override bool UnlockUser(string userName)
        {
            throw new NotSupportedException("The method or operation is not implemented.");
        }

    }
}
