using System;
using System.Data;
using System.Configuration;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

//Module to prevent a user in one virtual app from
//logging in against a user in a different virtual app
public class AuthenticatePortalID : IHttpModule
{
    //Urls are of the form http://some.server.name/portalid-a/default.aspx
    Regex r = new Regex("/portalid-(\\w+)", RegexOptions.IgnoreCase);

    public void Init(HttpApplication context)
    {
        context.PostAuthenticateRequest += new EventHandler(this.HandlePostAuthenticateRequest);
    }

    private void HandlePostAuthenticateRequest(object source, EventArgs e)
    {
        HttpApplication a = (HttpApplication)source;
        HttpContext c = a.Context;

        //If the portal ID in the forms auth ticket is for a different
        //portal than the one associated with this request - 
        //log the user out and redirect them to the login page.
        if (c.User.Identity.IsAuthenticated)
        {
            //Portal ID from the current request
            string portalID = (string)c.Items["portalID"];

            //Portal ID stored in the ticket
            FormsAuthenticationTicket ft = 
                ((FormsIdentity)c.User.Identity).Ticket;

            string portalIDFromTicket = ft.UserData;

            if (! (String.CompareOrdinal(portalID, portalIDFromTicket) == 0 ))
            {
                FormsAuthentication.SignOut();
                FormsAuthentication.RedirectToLoginPage();
            }
        }

    }

    public void Dispose() { }

}
