drop table ExtendedMembershipData
go

create table ExtendedMembershipData (
UserID uniqueidentifier NOT NULL PRIMARY KEY,
FirstName nvarchar(256) null,
LastName  nvarchar(256) null)
go

alter table ExtendedMembershipData add constraint
    FK_EMD_to_MembershipUser FOREIGN KEY(UserID) 
   references aspnet_Membership(UserId)
go

drop procedure CreateExtendedData
go
create procedure CreateExtendedData
@pApplicationName        nvarchar(256),
@pUserName               nvarchar(256),
@pFirstName              nvarchar(256),
@pLastName               nvarchar(256)
as

declare @ErrorCode int
declare @UserID  uniqueidentifier

select  @UserID = m.UserId
from    dbo.vw_aspnet_Applications a,
            dbo.vw_aspnet_Users u,
            dbo.vw_aspnet_MembershipUsers m
where   a.LoweredApplicationName     = LOWER(@pApplicationName)
and     a.ApplicationId              = u.ApplicationId
and     u.LoweredUserName            = LOWER(@pUserName)
and     u.UserId                     = m.UserId

if @UserID is null
    return -1

insert into dbo.ExtendedMembershipData values (@UserID, @pFirstName, @pLastName)
select @ErrorCode = @@ERROR

if (@ErrorCode <> 0)
    return -1
else
    return 0
go

drop procedure GetExtendedData
go
create procedure GetExtendedData
@pApplicationName        nvarchar(256),
@pUserName               nvarchar(256)
as

declare @UserID  uniqueidentifier

select  @UserID = m.UserId
from    dbo.vw_aspnet_Applications a,
        dbo.vw_aspnet_Users u,
        dbo.vw_aspnet_MembershipUsers m
where   a.LoweredApplicationName     = LOWER(@pApplicationName)
and     a.ApplicationId              = u.ApplicationId
and     u.LoweredUserName            = LOWER(@pUserName)
and     u.UserId                     = m.UserId

select FirstName, LastName
from   dbo.ExtendedMembershipData
where  UserID  = @UserID
go

drop procedure DeleteExtendedData
go
create procedure DeleteExtendedData
@pApplicationName        nvarchar(256),
@pUserName               nvarchar(256)
as

declare @UserID  uniqueidentifier

select  @UserID = m.UserId
from    dbo.vw_aspnet_Applications a,
        dbo.vw_aspnet_Users u,
        dbo.vw_aspnet_MembershipUsers m
where   a.LoweredApplicationName     = LOWER(@pApplicationName)
and     a.ApplicationId              = u.ApplicationId
and     u.LoweredUserName            = LOWER(@pUserName)
and     u.UserId                     = m.UserId

delete
from    dbo.ExtendedMembershipData
where   UserID     = @UserID
go

drop procedure UpdateExtendedData
go
create procedure UpdateExtendedData
@pApplicationName        nvarchar(256),
@pUserName               nvarchar(256),
@pFirstName              nvarchar(256),
@pLastName               nvarchar(256)
as

declare @UserID  uniqueidentifier
declare @ErrorCode int

select  @UserID = m.UserId
from    dbo.vw_aspnet_Applications a,
        dbo.vw_aspnet_Users u,
        dbo.vw_aspnet_MembershipUsers m
where   a.LoweredApplicationName     = LOWER(@pApplicationName)
and     a.ApplicationId              = u.ApplicationId
and     u.LoweredUserName            = LOWER(@pUserName)
and     u.UserId                     = m.UserId

update  dbo.ExtendedMembershipData
set     FirstName    = @pFirstName,
        LastName     = @pLastName
where   UserID     = @UserID
select @ErrorCode = @@ERROR
if (@ErrorCode <> 0)
    return -1
else
    return 0
go


--grant sproc rights
grant execute on CreateExtendedData to [NT AUTHORITY\NETWORK SERVICE]
grant execute on GetExtendedData to [NT AUTHORITY\NETWORK SERVICE]
grant execute on DeleteExtendedData to [NT AUTHORITY\NETWORK SERVICE]
grant execute on UpdateExtendedData to [NT AUTHORITY\NETWORK SERVICE]
go







