--Change the machine name REDBANK04 to the appropriate
--machine name for your server
SET NOCOUNT ON
go

use aspnetdb
go

truncate table StagingConfigurations
truncate table MachineApplicationToEnvironment
delete from Applications
delete from Machines
delete from Features
delete from StagingEnvironments
go

dbcc checkident('Applications',RESEED,0)
dbcc checkident('Machines',RESEED,0)
dbcc checkident('Features',RESEED,0)
dbcc checkident('StagingEnvironments',RESEED,0)
go

insert into StagingEnvironments values ('Development')
insert into StagingEnvironments values ('Test')
insert into StagingEnvironments values ('Staging')
insert into StagingEnvironments values ('Production')
go

insert into Features values ('Membership')
insert into Features values ('RoleManager')
insert into Features values ('ConnectionStrings')
go

insert into Machines values ('REDBANK04')
go

insert into Applications values ('/RedirectedRoleConfiguration')
insert into Applications values ('TestClientForRedirectedRoleConfig')
go

--Assign the app and the machine to the Test environment
insert into MachineApplicationToEnvironment values (1,1,2)
insert into MachineApplicationToEnvironment values (1,2,2)
go


--Store the role manager configuration for the test environment
--for the web app
insert into  StagingConfigurations values (1,2,3,
'  <connectionStrings>
    <clear />
    <add name="LocalSqlServer"
         connectionString="server=.;database=aspnetdb;Integrated Security=true" />
  </connectionStrings>
')
go

insert into  StagingConfigurations values (1,2,2,
'      <roleManager enabled="true">
        <providers>
          <clear/>
          <add name="AspNetSqlRoleProvider"
               type="System.Web.Security.SqlRoleProvider, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
               applicationName="RedirectedRoleConfiguration-Test"
               connectionStringName="LocalSqlServer"/>
        </providers>
      </roleManager>
')
go

--Store the membership configuration for the test environment
insert into  StagingConfigurations values (1,2,1,
'      <membership>
        <providers>
          <clear />
          <add name="AspNetSqlMembershipProvider"
               type="System.Web.Security.SqlMembershipProvider, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
               applicationName="RedirectedRoleConfiguration-Test"
               connectionStringName="LocalSqlServer"/>
        </providers>
      </membership>
')
go

--Store the role manager configuration for the test environment
--for the winforms app
insert into  StagingConfigurations values (2,2,3,
'  <connectionStrings>
    <clear />
    <add name="LocalSqlServer"
         connectionString="server=.;database=aspnetdb;Integrated Security=true" />
  </connectionStrings>
')
go

insert into  StagingConfigurations values (2,2,2,
'      <roleManager enabled="true">
        <providers>
          <clear/>
          <add name="AspNetSqlRoleProvider"
               type="System.Web.Security.SqlRoleProvider, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
               applicationName="RedirectedRoleConfiguration-Test"
               connectionStringName="LocalSqlServer"/>
        </providers>
      </roleManager>
')
go

--Store the membership configuration for the test environment
insert into  StagingConfigurations values (2,2,1,
'      <membership>
        <providers>
          <clear />
          <add name="AspNetSqlMembershipProvider"
               type="System.Web.Security.SqlMembershipProvider, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
               applicationName="RedirectedRoleConfiguration-Test"
               connectionStringName="LocalSqlServer"/>
        </providers>
      </membership>
')
go

--
--Create a different mapping for production and test for the web app
--
update MachineApplicationToEnvironment
set StagingEnvironmentID = 4 --production
--set StagingEnvironmentID = 2 --test
where MachineID = 1
and   ApplicationID = 1
go

insert into  StagingConfigurations values (1,4,3,
'  <connectionStrings>
    <clear />
    <add name="LocalSqlServer"
         connectionString="server=.;database=aspnetdb;Integrated Security=true" />
  </connectionStrings>
')
go

insert into  StagingConfigurations values (1,4,2,
'      <roleManager enabled="true">
        <providers>
          <clear/>
          <add name="AspNetSqlRoleProvider"
               type="System.Web.Security.SqlRoleProvider, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
               applicationName="RedirectedRoleConfiguration-Production"
               connectionStringName="LocalSqlServer"/>
        </providers>
      </roleManager>
')
go

insert into  StagingConfigurations values (1,4,1,
'      <membership>
        <providers>
          <clear />
          <add name="AspNetSqlMembershipProvider"
               type="System.Web.Security.SqlMembershipProvider, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
               applicationName="RedirectedRoleConfiguration-Production"
               connectionStringName="LocalSqlServer"/>
        </providers>
      </membership>
')
go

--
--Create a different mapping for production and test for the winforms app
--
update MachineApplicationToEnvironment
set StagingEnvironmentID = 4 --production
--set StagingEnvironmentID = 2 --test
where MachineID = 1
and   ApplicationID = 2
go

insert into  StagingConfigurations values (2,4,3,
'  <connectionStrings>
    <clear />
    <add name="LocalSqlServer"
         connectionString="server=.;database=aspnetdb;Integrated Security=true" />
  </connectionStrings>
')
go

insert into  StagingConfigurations values (2,4,2,
'      <roleManager enabled="true">
        <providers>
          <clear/>
          <add name="AspNetSqlRoleProvider"
               type="System.Web.Security.SqlRoleProvider, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
               applicationName="RedirectedRoleConfiguration-Production"
               connectionStringName="LocalSqlServer"/>
        </providers>
      </roleManager>
')
go

insert into  StagingConfigurations values (2,4,1,
'      <membership>
        <providers>
          <clear />
          <add name="AspNetSqlMembershipProvider"
               type="System.Web.Security.SqlMembershipProvider, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
               applicationName="RedirectedRoleConfiguration-Production"
               connectionStringName="LocalSqlServer"/>
        </providers>
      </membership>
')
go

--
--Test getting the configuration section back
--
exec GetConfiguration 'REDBANK04','/RedirectedRoleConfiguration','RoleManager' 
go
