using System;
using System.Security.Principal;
using System.Web;
using System.Web.Configuration;
using System.Web.UI;
using System.Web.Security;
using System.Collections;
using System.Collections.Generic;
using System.Web.Services;
using System.Web.Services.Protocols;


/// <summary>
/// Note:  Uses Membership static class right now - which implies the default provider.
/// </summary>
[WebService(Namespace = "http://microsoft.com/aspnet/prototype/membership")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
public class MembershipWebService : System.Web.Services.WebService {

    private bool isSecured = false;
    public WebServiceFormsTicket webServiceFormsTicket;
    private IPrincipal authenticatedPrincipal = new GenericPrincipal(new GenericIdentity(""), new string[0]);

    public MembershipWebService () 
    {
        string setting = WebConfigurationManager.AppSettings["secureWebService"];
        if (!String.IsNullOrEmpty(setting))
            isSecured = Boolean.Parse(setting);
    }

    [WebMethod]
    [SoapHeader("webServiceFormsTicket")]
    public WebServiceProxyableMembershipUser GetUserForAuthenticatedUser(bool userIsOnline)
    {
       GetAuthenticatedPrincipal();
       if (!authenticatedPrincipal.Identity.IsAuthenticated)
            return null;

        MembershipUser mu = GetProviderInstance().GetUser(authenticatedPrincipal.Identity.Name, userIsOnline);
        if (mu != null)
            return new WebServiceProxyableMembershipUser(mu);
        else
            return null;
    }

    [WebMethod]
    [SoapHeader("webServiceFormsTicket")]
    public WebServiceProxyableMembershipUser CreateUser(string username,
                                           string password,
                                           string email,
                                           string passwordQuestion,
                                           string passwordAnswer,
                                           bool isApproved,
                                           object providerUserKey)
    {
        if (isSecured)
        {
            GetAuthenticatedPrincipal();
            if (!authenticatedPrincipal.Identity.IsAuthenticated)
                return null;

            if (!authenticatedPrincipal.IsInRole("Administrators"))
                return null;
        }

        MembershipCreateStatus status;
        MembershipUser mu = GetProviderInstance().CreateUser(username,
                                           password,
                                           email,
                                           passwordQuestion,
                                           passwordAnswer,
                                           isApproved,
                                           providerUserKey,
                                           out status);

        if (mu != null)
            return new WebServiceProxyableMembershipUser(mu);
        else
            return null;
    }

    [WebMethod]
    [SoapHeader("webServiceFormsTicket")]
    public WebServiceProxyableMembershipUser GetUser(string username, bool userIsOnline)
    {
        if (isSecured)
        {
            GetAuthenticatedPrincipal();
            if (!authenticatedPrincipal.Identity.IsAuthenticated)
                return null;

            if (!authenticatedPrincipal.IsInRole("Administrators"))
                return null;
        }

        MembershipUser mu = GetProviderInstance().GetUser(username, userIsOnline);
        if (mu != null)
            return new WebServiceProxyableMembershipUser(mu);
        else
            return null;
    }

    [WebMethod]
    [SoapHeader("webServiceFormsTicket")]
    public void UpdateUser(WebServiceProxyableMembershipUser wsmu)
    {
        if (isSecured)
        {
            GetAuthenticatedPrincipal();
            if (!authenticatedPrincipal.Identity.IsAuthenticated)
                return;

            //Can only update the user object associated with the authenticated user
            if (!(String.Compare(authenticatedPrincipal.Identity.Name, wsmu.UserName, StringComparison.OrdinalIgnoreCase) == 0))
                return;

            //Make sure the provider associated with the logged in user
            //also matches the provider on the user object passed to this method
            MembershipProvider prov = GetProviderInstance();
            if (!(String.Compare(prov.Name, wsmu.ProviderName, StringComparison.OrdinalIgnoreCase) == 0))
                return;

            //Note that there is an edge case where the username string from the authenticated user
            //may match wsmu.UserName using an ordinal comparison.  But in the backend data store the two strings may be 
            //considered unique.  Use a binary collation order or Latin collation order in the database to prevent this.

        }

        GetProviderInstance().UpdateUser(wsmu);
        
    }

    [WebMethod]
    [SoapHeader("webServiceFormsTicket")]
    public WebServiceProxyableSearchResult FindUsersByName(string usernameToMatch, int pageIndex, int pageSize)
    {
        if (isSecured)
        {
            GetAuthenticatedPrincipal();
            if (!authenticatedPrincipal.Identity.IsAuthenticated)
                return new WebServiceProxyableSearchResult(new List<WebServiceProxyableMembershipUser>(),0);

            if (!authenticatedPrincipal.IsInRole("Administrators"))
                return new WebServiceProxyableSearchResult(new List<WebServiceProxyableMembershipUser>(), 0);
        }

        int totalRecords = 0;
        MembershipUserCollection muc =
            GetProviderInstance().FindUsersByName(usernameToMatch, pageIndex, pageSize, out totalRecords);

        List<WebServiceProxyableMembershipUser> proxyable = new List<WebServiceProxyableMembershipUser>();

        foreach (MembershipUser mu in muc)
        {
            proxyable.Add(new WebServiceProxyableMembershipUser(mu));
        }

        return new WebServiceProxyableSearchResult(proxyable, totalRecords);
    }

    [WebMethod]
    [SoapHeader("webServiceFormsTicket")]
    public WebServiceProxyableSearchResult GetAllUsers(int pageIndex, int pageSize)
    {
        if (isSecured)
        {
            GetAuthenticatedPrincipal();
            if (!authenticatedPrincipal.Identity.IsAuthenticated)
                return new WebServiceProxyableSearchResult(new List<WebServiceProxyableMembershipUser>(), 0);

            if (!authenticatedPrincipal.IsInRole("Administrators"))
                return new WebServiceProxyableSearchResult(new List<WebServiceProxyableMembershipUser>(), 0);
        }

        int totalRecords = 0;
        MembershipUserCollection muc =
            GetProviderInstance().GetAllUsers(pageIndex, pageSize, out totalRecords);

        List<WebServiceProxyableMembershipUser> proxyable = new List<WebServiceProxyableMembershipUser>();

        foreach (MembershipUser mu in muc)
        {
            proxyable.Add(new WebServiceProxyableMembershipUser(mu));
        }

        return new WebServiceProxyableSearchResult(proxyable, totalRecords);
    }

    [WebMethod]
    [SoapHeader("webServiceFormsTicket")]
    public bool ValidateUser(string username, string password)
    {
        //No security check since you need valid credentials to get a true response anyway
        return GetProviderInstance(username).ValidateUser(username, password);
    }

    //This overload should be called in cases where there is no other
    //place to look to determine the appropriate provider to use
    private MembershipProvider GetProviderInstance(string username)
    {
        //Add logic or raise event to determine the correct provider name
        //based on the username
        return Membership.Providers[Membership.Provider.Name];
    }

    private MembershipProvider GetProviderInstance()
    {
        if (authenticatedPrincipal.Identity.IsAuthenticated)
        {
            string providerName =
                ((FormsIdentity)authenticatedPrincipal.Identity).Ticket.UserData;
            return Membership.Providers[providerName];
        }
        else
        {
            if ((!isSecured) &&
                (this.webServiceFormsTicket != null) &&
                 (this.webServiceFormsTicket.ProviderName != null))
                return Membership.Providers[this.webServiceFormsTicket.ProviderName];
            else
                //Add logic or raise event to determine the correct provider name
                //based on the username
                return Membership.Providers[Membership.Provider.Name];
        }
    }

    private void GetAuthenticatedPrincipal()
    {
        if ( (this.webServiceFormsTicket == null) || (this.webServiceFormsTicket.Ticket == null))
        {
            authenticatedPrincipal = new GenericPrincipal(new GenericIdentity(""), new string[0]);
            return;
        }

        FormsAuthenticationTicket ft =
            FormsAuthentication.Decrypt(this.webServiceFormsTicket.Ticket);

        if (ft.Expired == true)
        {
            authenticatedPrincipal =  new GenericPrincipal(new GenericIdentity(""), new string[0]);
            return;
        }
        else
        {
            if (!Roles.Enabled)
                authenticatedPrincipal = new GenericPrincipal(new FormsIdentity(ft), new string[0]);
            else
            {
                authenticatedPrincipal = new RolePrincipal(new FormsIdentity(ft));
            }
        }
    }
    
}

