using System;
using System.Data;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.Xml.Serialization;
using System.Xml.Schema;

[XmlSchemaProvider("GetSchema")]
public class WebServiceProxyableMembershipUser : MembershipUser, IXmlSerializable
{
    private const string ns = "http://microsoft.com/aspnet/prototype/membership";
    private MembershipUser inner = null;

    //bootstrap constructor
    public WebServiceProxyableMembershipUser(MembershipUser mu)
    {
        if (mu != null)
        {
            inner = new MembershipUser(mu.ProviderName, mu.UserName, mu.ProviderUserKey, mu.Email, mu.PasswordQuestion,
                 mu.Comment, mu.IsApproved, mu.IsLockedOut, mu.CreationDate, mu.LastLoginDate,
                 mu.LastActivityDate, mu.LastPasswordChangedDate, mu.LastLockoutDate);
        }
    }

    //Required for XmlSerializer to work during deserialization
    public WebServiceProxyableMembershipUser() { }

    //Aggregation implementation
    public override string UserName
    {
        get { return inner.UserName; }
    }

    public override object ProviderUserKey
    {
        get { return inner.ProviderUserKey; }
    }


    public override string Email
    {
        get { return inner.Email; }
        set { inner.Email = value; }
    }


    public override string PasswordQuestion
    {
        get { return inner.PasswordQuestion; }
    }


    public override string Comment
    {
        get { return inner.Comment; }
        set { inner.Comment = value; }
    }


    public override bool IsApproved
    {
        get { return inner.IsApproved; }
        set { inner.IsApproved = value; }
    }

    public override bool IsLockedOut
    {
        get { return inner.IsLockedOut; }
    }

    public override DateTime LastLockoutDate
    {
        get { return inner.LastLockoutDate; }
    }


    public override DateTime CreationDate
    {
        get { return inner.CreationDate; }
    }


    public override DateTime LastLoginDate
    {
        get { return inner.LastLoginDate; }
        set { inner.LastLoginDate = value; }
    }


    public override DateTime LastActivityDate
    {
        get { return inner.LastActivityDate; }
        set { inner.LastActivityDate = value; }
    }


    public override DateTime LastPasswordChangedDate
    {
        get { return inner.LastPasswordChangedDate; }
    }

    //Not virtual in ASP.NET 2.0
    new public bool IsOnline
    {
        get
        {
            TimeSpan ts = new TimeSpan(0, Membership.UserIsOnlineTimeWindow, 0);
            DateTime dt = DateTime.UtcNow.Subtract(ts);
            return LastActivityDate.ToUniversalTime() > dt;
        }
    }

    public override string ToString()
    {
        return inner.UserName;
    }

    public override string ProviderName
    {
        get { return inner.ProviderName; }
    }

    public override string GetPassword()
    {
        return inner.GetPassword();
    }

    public override string GetPassword(string passwordAnswer)
    {
        return inner.GetPassword(passwordAnswer);
    }

    public override bool ChangePassword(string oldPassword, string newPassword)
    {
        return inner.ChangePassword(oldPassword, newPassword);
    }

    public override bool ChangePasswordQuestionAndAnswer(string password, string newPasswordQuestion, string newPasswordAnswer)
    {
        return inner.ChangePasswordQuestionAndAnswer(password, newPasswordQuestion, newPasswordAnswer);
    }

    public override string ResetPassword(string passwordAnswer)
    {
        return inner.ResetPassword(passwordAnswer);
    }

    public override string ResetPassword()
    {
        return inner.ResetPassword();
    }

    public virtual bool UnlockUser()
    {
        return inner.UnlockUser();
    }

    //Per docs
    public System.Xml.Schema.XmlSchema GetSchema()
    {
        throw new NotSupportedException("Use the provider attribute instead");
    }

    // This is the method named by the XmlSchemaProviderAttribute applied to the type.
    public static XmlQualifiedName GetSchema(XmlSchemaSet xs)
    {
        string fullvpath = VirtualPathUtility.Combine(
            VirtualPathUtility.AppendTrailingSlash(HttpRuntime.AppDomainAppVirtualPath), "App_Code\\WebServiceProxyableMembershipUser.xsd.exclude");
        string fullphyspath = HttpContext.Current.Server.MapPath(fullvpath);

        XmlSerializer schemaSerializer = new XmlSerializer(typeof(XmlSchema));
        using (XmlTextReader xr = new XmlTextReader(fullphyspath))
        {
            XmlSchema s = (XmlSchema)schemaSerializer.Deserialize(xr,null);

            //Associate the newly loaded schema with the schema set
            xs.XmlResolver = new XmlUrlResolver();
            xs.Add(s);

            return new XmlQualifiedName("WebServiceProxyableMembershipUser", ns);
        }
    }

    //Since custom serialization is used, custom deserialization is needed as well
    public void ReadXml(System.Xml.XmlReader reader)
    {
        //Position to the first piece of the serialized MembershipUser
        reader.MoveToContent();
        bool found = reader.ReadToDescendant("userName",ns);

        string userName = reader.ReadElementString("userName",ns);
        string providerUserKey = reader.ReadElementString("providerUserKey", ns);
        string email = reader.ReadElementString("email", ns);
        string passwordQuestion = reader.ReadElementString("passwordQuestion", ns);
        string comment = reader.ReadElementString("comment", ns);
        bool isApproved = reader.ReadElementContentAsBoolean("isApproved", ns);
        bool isLockedOut = reader.ReadElementContentAsBoolean("isLockedOut", ns);
        DateTime lastLockoutDate = reader.ReadElementContentAsDateTime("lastLockoutDate", ns);
        DateTime creationDate = reader.ReadElementContentAsDateTime("creationDate", ns);
        DateTime lastLoginDate = reader.ReadElementContentAsDateTime("lastLoginDate", ns);
        DateTime lastActivityDate = reader.ReadElementContentAsDateTime("lastActivityDate", ns);
        DateTime lastPasswordChangedDate = reader.ReadElementContentAsDateTime("lastPasswordChangedDate", ns);
        bool isOnline = reader.ReadElementContentAsBoolean("isOnline", ns);
        string providerName = reader.ReadElementString("providerName", ns);

        inner = new MembershipUser(providerName, userName, providerUserKey, email, passwordQuestion,
                                   comment, isApproved, isLockedOut, creationDate, lastLoginDate,
                                   lastActivityDate, lastPasswordChangedDate, lastLockoutDate);
    }

    //Custom serialization to handle read-only properties
    public void WriteXml(System.Xml.XmlWriter writer)
    {
        //writer.WriteStartElement("membershipUser");
        writer.WriteElementString("userName", UserName);
        writer.WriteElementString("providerUserKey", ProviderUserKey.ToString());
        writer.WriteElementString("email", Email);
        writer.WriteElementString("passwordQuestion", PasswordQuestion);
        writer.WriteElementString("comment", Comment);
        writer.WriteElementString("isApproved", ConvertToBool(IsApproved));
        writer.WriteElementString("isLockedOut", ConvertToBool(IsLockedOut));
        writer.WriteElementString("lastLockoutDate", ConvertToXmlDateTime(LastLockoutDate));
        writer.WriteElementString("creationDate", ConvertToXmlDateTime(CreationDate));
        writer.WriteElementString("lastLoginDate", ConvertToXmlDateTime(LastLoginDate));
        writer.WriteElementString("lastActivityDate", ConvertToXmlDateTime(LastActivityDate));
        writer.WriteElementString("lastPasswordChangedDate", ConvertToXmlDateTime(LastPasswordChangedDate));
        writer.WriteElementString("isOnline", ConvertToBool(IsOnline));
        writer.WriteElementString("providerName", ProviderName);
        //writer.WriteEndElement();
    }

    //Helper methods to deal with XmlSerialized datetime values
    private string ConvertToBool(Boolean param)
    {
        return (param == true ? "true" : "false");
    }

    private string ConvertToXmlDateTime(DateTime dt)
    {
        XmlSerializer xs = new XmlSerializer(typeof(DateTime));
        StringWriter sw = new StringWriter();
        xs.Serialize(sw, dt);

        XmlDocument xd = new XmlDocument();
        xd.LoadXml(sw.ToString());
        return xd.SelectSingleNode("/dateTime").InnerText;

    }
}
